module I2S_master
	#(parameter width = 16)(
	output [width-1:0] received_word,
	input [width-1:0] transmit_word,
	output transmit_empty_flag,
	input transmit_load,
	output i2s_fs,
	output i2s_clk,
	input i2s_rx,
	output i2s_tx,
	input clk
);

	logic next_transmit_empty_flag, next_i2s_fs, rx_bit, next_rx_bit;
	logic i2sclk, next_i2sclk;
	logic [3:0] ctr, next_ctr;
	logic [width-1:0] sr_tx, next_sr_tx, sr_rx, next_sr_rx,
		receive_register, next_receive_register;
	
	typedef enum{IDLE, START, FS, FSEND, SHIFT, END} STATE;
	STATE state, next_state;
	
	initial begin
		i2s_fs = 0;
		transmit_empty_flag = 1;
		state = IDLE;
	end
		
	assign i2s_clk = i2sclk;	
	assign i2s_tx = sr_tx[15];
	assign received_word = receive_register;
	
	always_ff@(posedge clk) begin	
		rx_bit <= next_rx_bit;
		ctr <= next_ctr;
		i2s_fs <= next_i2s_fs;
		receive_register <= next_receive_register;
		transmit_empty_flag <= next_transmit_empty_flag;
		sr_rx <= next_sr_rx;
		sr_tx <= next_sr_tx;
		i2sclk <= next_i2sclk;
		state <= next_state;
	end
	
	always_comb begin
		next_rx_bit <= rx_bit;
		next_ctr <= ctr;
		next_i2s_fs <= i2s_fs;
		next_receive_register <= receive_register;
		next_transmit_empty_flag <= transmit_empty_flag;
		next_sr_rx <= sr_rx;
		next_sr_tx <= sr_tx;
		next_i2sclk <= ~i2sclk;
		next_state <= state;
		
		if (i2sclk == 0) next_rx_bit <= i2s_rx;
		
		case (state) 
		
			IDLE : begin
				next_transmit_empty_flag <= 1;
				if (transmit_load) begin
					next_i2sclk <= 1;
					next_sr_tx <= transmit_word;
					next_transmit_empty_flag <= 0;
					next_state <= START;
				end
			end
			
			START: begin						// start FS
				if (i2sclk == 0) begin 
					next_i2s_fs <= 1; 
					next_state <= FS;
				end
			end
			
			FS: begin  							// continue FS
				if (i2sclk == 0) begin 
					next_state <= FSEND;
				end
			end
			
			FSEND : begin            		// hold CLK low and start data
				next_ctr <= 4'b1111;
				next_i2sclk <= 1; 
				next_i2s_fs <= 0;
				next_state <= SHIFT;
			end
			
			SHIFT: begin
				if (i2sclk == 0) begin		// shift data out and in
					next_sr_rx <= {sr_rx[width-2:0], rx_bit};
					next_sr_tx <= {sr_tx[width-2:0], 1'b0};
					next_ctr <= ctr-4'b0001;
					if (ctr == 0) next_state <= END;
				end
			end
			
			END: begin							// read last bit and end transfer
				if (i2sclk == 0) begin 
					next_sr_tx <= {sr_tx[width-2:0], 1'b0};  // set output low
					next_receive_register <= {sr_rx[width-2:0], rx_bit};
					next_state <= IDLE;
				end
			end
		
		endcase
	
	end
		
endmodule