// File name: SpiDeviceV01.sv
//
// 23Jul2012 .. version started .. K.Metzger
//
module SpiSlave
	#(parameter width = 16)(
	output [width-1:0] received_word,
	output received_flag,
	input received_flag_clear,
	input [width-1:0] transmit_word,
	output transmit_empty_flag,
	input transmit_load,
	input spi_fs,
	input spi_clk,
	input spi_rx,
	output spi_tx
);

	logic [width-1:0] sr_tx, sr_rx, transmit_register, receive_register;
	
	// implement the transmiter
	
	always@(posedge spi_clk or posedge spi_fs) begin
		if(spi_fs) sr_tx <= transmit_register;
		else sr_tx <= {sr_tx[width-2:0], 1'b0};
	end
	
	always@(negedge spi_fs or posedge transmit_load) begin
		if(transmit_load) begin
			transmit_empty_flag <= 0;
			transmit_register <= transmit_word;
		end
		else transmit_empty_flag <= 1;
	end
	
	assign spi_tx = sr_tx[width-1];  // shifting msb out   

	// implement the receiver
	
	always@(negedge spi_clk) sr_rx[width-1:0] <= {sr_rx[width-2:0], spi_rx};
	always@(posedge spi_fs) receive_register <= sr_rx;
	always@(posedge spi_fs or posedge received_flag_clear) begin
		if(received_flag_clear) received_flag <= 0;
		else received_flag <= 1;
	end
	assign received_word = receive_register;
	
endmodule